// $Id: AnalogClock.cc,v 1.3 2001/03/03 22:02:34 rob Exp $

#include "AnalogClock.h"

// Combined sin & cos table.  Cosine is offset by index+30 in the table.
// Values are *1000, and the angles are in .5 minute increments.
short sincos[] = {
  0, 52, 104, 156, 207, 258, 309, 358, 406, 453,
  500, 544, 587, 629, 669, 707, 743, 777, 809, 838,
  866, 891, 913, 933, 951, 965, 978, 987, 994, 998,
  999, 998, 994, 987, 978, 965, 951, 933, 913, 891,
  866, 838, 809, 777, 743, 707, 669, 629, 587, 544,
  499, 453, 406, 358, 309, 258, 207, 156, 104, 52,
  0, -52, -104, -156, -207, -258, -309, -358, -406, -453,
  -500, -544, -587, -629, -669, -707, -743, -777, -809, -838,
  -866, -891, -913, -933, -951, -965, -978, -987, -994, -998,
  -999, -998, -994, -987, -978, -965, -951, -933, -913, -891,
  -866, -838, -809, -777, -743, -707, -669, -629, -587, -544,
  -499, -453, -406, -358, -309, -258, -207, -156, -104, -52,
  0, 52, 104, 156, 207, 258, 309, 358, 406, 453,
  500, 544, 587, 629, 669, 707, 743, 777, 809, 838,
  866, 891, 913, 933, 951, 965, 978, 987, 994, 998
};


AnalogClock::AnalogClock(Container *parent, int tc): Clock(parent,tc) {
  dprintf("in AnalogClock::AnalogClock\n");

  hourOffset=0;
  minuteOffset=0;
}

void AnalogClock::setPreferredSize(int dir) 
{
  dprintf("in AnalogClock::setPreferredSize\n");

  if (dir==VERTICAL) {
    h=w;
  }
  else {
    w=h;
  }
  center.x = (w >> 1); // pick up possible resize changes
  center.y = (h >> 1);
}

void AnalogClock::drawFace() 
{
  XSegment lines[100];
  XSegment *l=lines;
  int numsegments=0;
	
  int angle1, angle2;
	
  for (int i = 0; i < 60; i += 5) {
    angle1 = sincos[i * 2] * w;
    angle2 = -sincos[i * 2 + 30] * h;
    l->x1 = angle1 * 9 / 20000 + center.x;
    l->y1 = angle2 * 9 / 20000 + center.y;
    l->x2 = angle1 * 10 / 20000 + center.x;
    l->y2 = angle2 * 10 / 20000 + center.y;
    numsegments++;
    l++;
  }
  XDrawSegments(display, win, drawGC, lines, numsegments);
}

void AnalogClock::drawHands(GC gc) 
{
  XSegment lines[100];
  XSegment *l=lines;
  int numsegments=0;
  XPoint HourHand, MinuteHand;
  int HourAngle, MinuteAngle;
	
  HourAngle = hour * 10 + minute / 6;
  MinuteAngle = minute * 2;
	
  HourHand.x = sincos[HourAngle] * w * 60 / 200000 + center.x;
  HourHand.y = -sincos[HourAngle + 30] * h * 60 / 200000 + center.y;
  MinuteHand.x = sincos[MinuteAngle] * w * 85 / 200000 + center.x;
  MinuteHand.y = -sincos[MinuteAngle + 30] * h * 85 / 200000 + center.y;
	
  for (int i = -1; i < 2; i++) {
    for (int j = -1; j < 2; j++) {
      l->x1 = center.x + i;
      l->y1 = center.y + j;
      l->x2 = HourHand.x;
      l->y2 = HourHand.y;
      numsegments++;
      l++;
      l->x1 = center.x + i;
      l->y1 = center.y + j;
      l->x2 = MinuteHand.x;
      l->y2 = MinuteHand.y;
      numsegments++;
      l++;
    }
  }
  XDrawSegments(display, win, gc, lines, numsegments);
}


// Overriding Container::paint to draw a clock face
void AnalogClock::paint()
{
  dprintf("in AnalogClock::paint\n");

  int ck = hour * 60 + minute; // FIXME: should be checksum to see if hands
// have moved, but once we get into paint, the new time has already been
// calculated.

  int full=TRUE;
	
    /* draw the clock face if this is a complete redraw */
  if (full) {
    drawFace();
  }
	
    /* draw the hands if they have moved */
  if (full || (ck != checksum)) {
    drawHands(drawGC);
  }
}
