// $Id: BarGraph.cc,v 1.11 2001/03/28 02:29:42 rob Exp $

#include "BarGraph.h"

// Generic baseclass for horizontal bar graphs.

BarGraph::BarGraph(Container *parent, int tc): TimedContainer(parent,tc) {
  dprintf("in BarGraph::BarGraph\n");

  this->colorList=NULL;

  setRange(0,100);
  setScheme(COLOR_SCHEME_GREEN);
}

BarGraph::BarGraph(Container *parent, int tc, int min, int max, char *label): TimedContainer(parent,tc) {
  dprintf("in BarGraph::BarGraph\n");

  setRange(min,max);

  this->label=label;
  this->colorList=NULL;

  setScheme(COLOR_SCHEME_GREEN);
}

BarGraph::BarGraph(Container *parent, int tc, int min, int max, char *label, int scheme): TimedContainer(parent,tc) {
  dprintf("in BarGraph::BarGraph\n");

  setRange(min,max);

  this->label=label;
  this->colorList=NULL;

  setScheme(scheme);
}

void BarGraph::addColor(int breakpoint, char *name) {
  BarColor *c=(BarColor *)malloc(sizeof(BarColor));
  string *s=new string(name);
	
  c->breakpoint=breakpoint;
  c->colorName=s;
  c->next=NULL;
	
  BarColor *color=colorList;
  if (!color) {
    colorList=c;
  }
  else {
    while (color->next) {
      color=color->next;
    }
    color->next=c;
  }
}

void BarGraph::setScheme(int scheme) {
  colorScheme=scheme;
}

void BarGraph::allocScheme() {
  switch (colorScheme) {
    case COLOR_SCHEME_RYG:
      addColor((25*max/100)-min,"IndianRed");
      addColor((40*max/100)-min,"LightGoldenrod1");
      addColor(max-min,"Aquamarine2");
      break;
    case COLOR_SCHEME_GYR:
      addColor((60*max/100)-min,"Aquamarine2");
      addColor((85*max/100)-min,"LightGoldenrod1");
      addColor(max-min,"IndianRed");
      break;
    case COLOR_SCHEME_GREEN:
    default:
      addColor(max-min,"Aquamarine2");
      break;
    case COLOR_SCHEME_CUSTOM:
      break;
  }
}

void BarGraph::allocColors() {
  XGCValues gcv;

  allocScheme();
  BarColor *c=colorList;

  while (c) {
    string *s=c->colorName;
    const char *str=s->c_str();
    getColor(&c->color,str);
		
    gcv.foreground = c->color.pixel;
    gcv.background = background.pixel;
    gcv.function = GXcopy;
    c->gc = XCreateGC(display, win,
      GCForeground | GCBackground | GCFunction,
      &gcv);

    c=c->next;
  }
}


void BarGraph::realize() {
  dprintf("in BarGraph::realize\n");

  allocColors();
  TimedContainer::realize();
}

void BarGraph::setPreferredSize(int dir) {
  dprintf("in BarGraph::setPreferredSize\n");

  if (dir==VERTICAL) {
    h=font_height+4*BORDER_WIDTH;
  }
  else {
    w=h;
  }

  bar.y=BORDER_WIDTH;
  bar.h=h-2*BORDER_WIDTH;
}

// Bar graphs are drawn when the current value is between min and max.
void BarGraph::setRange(int min, int max) {
  dprintf("in BarGraph::setRange\n");

  this->min=min;
  if (max<=min) max=min+1;
  this->max=max;
  range=max-min;
	
  bar.cur=min-1; // guarantee that no bar is drawn to start
  bar.last_x=0; // force redraw of entire bar at next update
}

void BarGraph::setLabel(char *label) {
  this->label=label;
}

// the bar is always draw with the minimum value on the left side of the
// window, and as the current value increases, the bar grows to the right.
void BarGraph::drawBar(BarInfo *b) {
  dprintf("in BarGraph::drawBar\n");

  int val=b->cur;
  if (val>max) val=max;
  val-=min;
	
  int new_x=(w*val)/range; // pixel position

  if (new_x>b->last_x) {
    int width=new_x-b->last_x;
    BarColor *c=colorList;
    while (c->next && val>c->breakpoint) c=c->next;
		
    XFillRectangle(display, win, c->gc, b->last_x, b->y, width, b->h);
  }
  else if (new_x<b->last_x) {
    int width=b->last_x-new_x;
    XClearArea(display, win, new_x, b->y, width, b->h, FALSE);
  }
  b->last_x=new_x;

  if (!label.empty()) {
    XDrawString(display, win, drawGC, 2, font_ascent+1+BORDER_WIDTH,
      label.c_str(), label.size());
  }
}

void BarGraph::prepare() {
  dprintf("in BarGraph::prepare\n");

  bar.cur=((bar.cur+7)*3)%range;
}

void BarGraph::paint() {
  dprintf("in BarGraph::paint\n");
	
  bar.last_x=0;
  drawBar(&bar);
}

void BarGraph::update() {
  dprintf("in BarGraph::update\n");
	
  drawBar(&bar);
}
