// $Id: Battery.cc,v 1.5 2001/03/03 22:02:34 rob Exp $

#include "Battery.h"

Battery::Battery(Container *parent, int tc): BarGraph(parent,tc) {
  dprintf("in Battery::Battery\n");

  setRange(0,100);
  setLabel("Battery");
  setScheme(COLOR_SCHEME_RYG);

  mode=-1;
  batt_start=time(NULL);
}

void Battery::prepare() {
  dprintf("in Battery::prepare\n");

  bar.cur=getPercentRemaining();
}


  /* Arguments, with symbols from linux/apm_bios.h.  Information is
     from the Get Power Status (0x0a) call unless otherwise noted.

     Linux version 0.8
     
     0) Linux driver version (this will change if format changes)
     1) APM BIOS Version.  Usually 1.0 or 1.1.
     2) APM flags from APM Installation Check (0x00):
          bit 0: APM_16_BIT_SUPPORT
          bit 1: APM_32_BIT_SUPPORT
          bit 2: APM_IDLE_SLOWS_CLOCK
          bit 3: APM_BIOS_DISABLED
          bit 4: APM_BIOS_DISENGAGED
     3) AC line status
          0x00: Off-line
          0x01: On-line
          0x02: On backup power (APM BIOS 1.1 only)
          0xff: Unknown
     4) Battery status
          0x00: High
          0x01: Low
          0x02: Critical
          0x03: Charging
          0xff: Unknown
     5) Battery flag
          bit 0: High
          bit 1: Low
          bit 2: Critical
          bit 3: Charging
          bit 7: No system battery
          0xff: Unknown
     6) Remaining battery life (percentage of charge):
          0-100: valid
          -1: Unknown
     7) Remaining battery life (time units):
          Number of remaining minutes or seconds
          -1: Unknown
     8) min = minutes; sec = seconds
  */

// integer percentage (0-100) of battery life remaining.
int Battery::getPercentRemaining() {
  int perc=0;
  char buf[128];
	
  int fd = open("/proc/apm", O_RDONLY);
  if (fd>0) {
    ssize_t len=read(fd,buf,sizeof(buf)-1);
    buf[len]=(char)0;
    close(fd);

    if (len>0) {
      float dum;
      int i,ac,batt,amount;
      char units[16];
			
      sscanf(buf,"%f %f %x %x %x %x %d%% %d %15s",
        &dum,&dum,&i,&ac,&batt,&i,&perc,&amount,units);
			
      if (perc>100) perc=100;
      if (perc<0) perc=0;

      dprintf(" fd=%d ac=%d battery=%d perc=%d time left=%d %s\n",
        fd,ac,batt,perc,amount,units);

      time_t curr_time=time(NULL);
      if (batt==3) {
        if (mode!=CHARGING) {
          setLabel("Charging");
          mode=CHARGING;
        }
        batt_start=curr_time;
      }
      else if (ac!=1) {
        int minutes=(int)(curr_time-batt_start)/60;
        int hours=minutes/60;
        char label[32];
				
        sprintf(label,"%.1d:%.2d Battery",hours,minutes%60);
        dprintf(" %s\n",label);
        setLabel(label);
        mode=BATTERY;
      }
      else {
        if (mode!=AC_POWER) {
          setLabel("AC Power");
          mode=AC_POWER;
        }
        perc=0;
        batt_start=curr_time;
      }
    }
  }

  return perc;
}
