// $Id: CPU.cc,v 1.7 2001/03/04 03:26:33 rob Exp $

#include "CPU.h"

// constructor for instantaneous CPU usage of individual processors.  This
// usage percentage is actually calculated from the idle time read from
// /proc/stat (on Linux)

CPU::CPU(Container *parent, int tc): BarGraph(parent,tc) {
  dprintf("in CPU::CPU\n");

  char label[8];
  strcpy(label,"cpu ");
  this->cpuTag=label;
  setLabel(label);

  last_user=-1;
  idle_perc=1.0;
}

CPU::CPU(Container *parent, int tc, int cpuNum): BarGraph(parent,tc) {
  dprintf("in CPU::CPU\n");

  char label[8];
  if (cpuNum>=0) {
    sprintf(label,"cpu%d",cpuNum);
    this->cpuTag=label;
  }
  else {
    strcpy(label,"cpu ");
    this->cpuTag=label;
  }
  setLabel(label);

  last_user=-1;
  idle_perc=1.0;
}

// CPU usage is 100% - idle percentage
void CPU::prepare() {
  dprintf("in CPU::prepare (%s)\n",cpuTag.c_str());

  calculateInfo();
  bar.cur=(int)(100*(1.0-idle_perc));
}


void CPU::calculateInfo() {
  char buf[512];
	
  int fd = open("/proc/stat", O_RDONLY);
  if (fd>0) {
    ssize_t len=read(fd,buf,sizeof(buf)-1);
    buf[len]=(char)0;

    char *str=strstr(buf,cpuTag.c_str());
    if (str) {
      str+=strlen(cpuTag.c_str());
			
      sscanf(str,"%ld %ld %ld %ld",&user,&nice,&sys,&idle);
      if (last_user>0) {
        float total=idle-last_idle + sys-last_sys + nice-last_nice +
          user-last_user;
        idle_perc=(idle-last_idle)/total;
        user_perc=(user-last_user)/total;
        sys_perc=(sys-last_sys)/total;
        nice_perc=(nice-last_nice)/total;
      }
      last_idle=idle;
      last_sys=sys;
      last_nice=nice;
      last_user=user;
    }
    else {
      dprintf("CPU: bad string search\n");
      idle_perc=1.0;
      user_perc=sys_perc=nice_perc=0.0;
    }
    close(fd);
  }
  else {
    idle_perc=1.0;
    user_perc=sys_perc=nice_perc=0.0;
  }
}

// search through the list of cpus in /proc/stat, and create a Container
// object for each of them.
void CPU::create(Container *parent, int tc) {
  char buf[1024];
	
  int fd = open("/proc/stat", O_RDONLY);
  if (fd>0) {
    ssize_t len=read(fd,buf,sizeof(buf)-1);
    buf[len]=(char)0;
    close(fd);

    char *str=buf;
      // first, search for multiprocessor systems
    if ((str=strstr(buf,"cpu0"))!=NULL) {
      while ((str=strstr(str,"cpu"))!=NULL) {
        int cpu=0;
        str+=3;
        while (*str && isdigit(*str)) cpu=cpu*10+(int)(*str++)-(int)'0';
				
        dprintf("CPU::create: found cpu %d\n",cpu);

          // ignore return value because we never destroy
        new CPU(parent,tc,cpu);
      }
    }
    else {
        // ignore return value because we never destroy
      new CPU(parent,tc);
    }
  }
}
