// $Id: Calendar.cc,v 1.6 2001/03/20 04:16:24 rob Exp $

#include "Calendar.h"

// This is a popup window (a "tooltip") that displays a calendar, highlighting
// the current date.  In the future, I might have it display two or more
// months in the popup...

Calendar::Calendar(): Tooltip() {
  dprintf("in Calendar::Calendar\n");

  cell_padding=1;
  border_width=2;
  months_to_display=2;
  highlightColorName="Aquamarine2";
  const char *c=highlightColorName.c_str();
  highlightGC=getGC(c);
}

void Calendar::setPreferredSize(int dir) {
  dprintf("in Calendar::setPreferredSize\n");

  cell_height=font_height + 2*cell_padding+1;
  h=cell_height*8*months_to_display + 2*border_width;

  cell_width=font_width*2 + 2*cell_padding+1;
    // the +1 at the end of the width calculation is for the one extra line
    // at the end; i.e. for 7 cells, there are 8 lines
  w=cell_width*7 + 1 + 2*border_width;
}

int Calendar::getDaysInMonth(int month, int year) {
  static int DaysInMonth[] = {31,28,31,30,31,30,31,31,30,31,30,31};

  int days = DaysInMonth[month];
  if (month == 1) {	/* Check for February */
    if ((year % 4 == 0) &&
      (((year % 100 != 0) || (year % 1000 == 0))))
      days++;
  }
  return days;
}

char Calendar::getDayAbbrev(int day) {
  static char DayAbbrev[] = {'S','M','T','W','T','F','S'};
	
  return DayAbbrev[day];
}

void Calendar::drawCal(struct tm *tmval, int y_offset, bool draw_today) {
  int i;
  XSegment lines[100], *l;
  int numsegments;

  int StartDayOfMonth,	/* 0-6, first day of week in the month */
    LastDayOfMonth,	/* 0-6, last day of week in month */
    NumDays,		/* number days in month */
    WeekLines,		/* number of lines of boxes required to print calendar */
    StartWeek, LastWeek, CurrentWeek, day, week;
  numsegments = 0;
  l = lines;

  char tmp[32];

  NumDays = getDaysInMonth(tmval->tm_mon,tmval->tm_year);

    /* avoid getting a negative number for StartDayOfMonth */
  StartDayOfMonth = ((7*5) - ((tmval->tm_mday-1) - tmval->tm_wday)) % 7;
	
  LastDayOfMonth = (StartDayOfMonth + NumDays - 1) % 7;
  i = StartDayOfMonth + NumDays;
  WeekLines = i / 7;
  if (i % 7 > 0)
    WeekLines++;
  if (WeekLines == 5)
    StartWeek = 0, LastWeek = 4;
  else if (WeekLines == 6)
    StartWeek = 0, LastWeek = 5;
  else
    StartWeek = 1, LastWeek = 4;
  CurrentWeek = StartWeek + (StartDayOfMonth + tmval->tm_mday - 1) / 7;
	
  dprintf("in Calendar::drawCal:  %d/%d:  DayOfWeek=%d NumDays=%d\n"
    "     StartDayOfMonth=%d LastDayOfMonth=%d\n"
    "     WeekLines=%d StartWeek=%d LastWeek=%d CurrentWeek=%d\n",
    tmval->tm_mday,tmval->tm_mon,tmval->tm_wday,
    NumDays,StartDayOfMonth,LastDayOfMonth,
    WeekLines,StartWeek,LastWeek,CurrentWeek);
	
  int xoff = border_width;
  int cell_width = font_width*2+2*cell_padding+1; // including linewidth

  int yoff = y_offset + border_width + font_height;
  int cell_height = font_height+2*cell_padding+1;
	
  strftime(tmp, 32, "%B %Y", tmval);
  drawCenteredText(drawGC,w/2,yoff,tmp,strlen(tmp));
  yoff+=cell_height;

    // draw abbreviations for day names
  for (i = 0; i < 7; i++) {
    int x1=xoff + cell_width*i + cell_width/2 + 1;
    char c=getDayAbbrev(i);
    drawCenteredText(drawGC, x1, yoff-1, &c, 1);
  }

    // draw vertical lines
  for (i = 0; i <= 7; i++) {
    int topoff, botoff;
		
    l->x1 = l->x2 = xoff + cell_width*i;
    topoff = StartWeek;
    if (i < StartDayOfMonth)
      topoff++;
    l->y1 = yoff + cell_height*topoff;
    botoff = LastWeek + 1;
    if (i > LastDayOfMonth + 1)
      botoff--;
    l->y2 = yoff + cell_height*botoff;
    numsegments++;
    l++;
  }
    // draw full horizontal lines
  for (i = StartWeek + 1; i <= LastWeek; i++) {
    l->y1 = l->y2 = yoff + cell_height*i;
    l->x1 = xoff;
    l->x2 = xoff + cell_width*7;
    numsegments++;
    l++;
  }
  l->y1 = l->y2 = yoff + cell_height*StartWeek;
  l->x1 = xoff + cell_width*StartDayOfMonth;
  l->x2 = xoff + cell_width*7;
  numsegments++;
  l++;
  l->y1 = l->y2 = yoff + cell_height*(LastWeek + 1);
  l->x1 = xoff;
  l->x2 = xoff + cell_width*(LastDayOfMonth + 1);
  numsegments++;
  l++;
  XDrawSegments(display, win, drawGC, lines, numsegments);
	
    // draw the numbers showing the days of the month
  day = StartDayOfMonth;
  week = StartWeek;
  for (i = 1; i <= NumDays; i++) {
    int x1, y1;
		
      //x1=xoff + cell_width*day + cell_padding + 1;
      //if (i<10) x1+=(cell_width/4);
    x1=xoff + cell_width*day + cell_width/2 + 1;
    y1=yoff + cell_height*week + font_ascent + cell_padding + 1;

    if (draw_today && i==tmval->tm_mday) {
      XFillRectangle(display, win, highlightGC,
        xoff + cell_width*day + 1, yoff + cell_height*week + 1,
        cell_width-1, cell_height-1);
    }
		
    sprintf(tmp,"%d",i);
      //XDrawString(display, win, drawGC, x1, y1, tmp, strlen(tmp));
    drawCenteredText(drawGC, x1, y1, tmp, strlen(tmp));
    day++;
    if (day > 6) {
      week++;
      day = 0;
    }
  }
}

void Calendar::paint() {
  dprintf("in Calendar::paint\n");
  int y_offset=0;
	
    /* get the current time */
  time_t t=time(NULL);
  struct tm *current = localtime(&t);
  drawCal(current,y_offset,true);

  int future_months=months_to_display-1;
  while (future_months-->0) {
      // get pointer to sometime in the first day of the month
    t-=(current->tm_mday-1)*24*60*60;

      // add number of seconds in current month to get pointer to
      // some time in the first day of next month
    t+=getDaysInMonth(current->tm_mon,current->tm_year)*24*60*60;
    current=localtime(&t);
    y_offset+=cell_height*8;
    drawCal(current,y_offset,false);
  }
}
