// $Id: Chart.cc,v 1.5 2001/03/03 22:02:34 rob Exp $

#include "Chart.h"

Chart::Chart(Container *parent, int tc): TimedContainer(parent,tc) {
  dprintf("in Chart::Chart\n");

  this->valuePerZoom=100;
  this->chartColorName="Aquamarine4";
  this->zoomColorName="Aquamarine2";
}

Chart::Chart(Container *parent, int tc, int max): TimedContainer(parent,tc) {
  dprintf("in Chart::Chart\n");

  this->valuePerZoom=max;
  this->chartColorName="Aquamarine4";
  this->zoomColorName="Aquamarine2";
}

void Chart::allocChart() {
  buffer=(int *)calloc(w,sizeof(int));
  store=0;

  const char *str=chartColorName.c_str();
  chartGC=getGC(str);
  str=zoomColorName.c_str();
  zoomGC=getGC(str);
}


void Chart::realize() {
  dprintf("in Chart::realize\n");

  allocChart();
  TimedContainer::realize();
}

void Chart::setPreferredSize(int dir) {
  dprintf("in Chart::setPreferredSize\n");

  if (dir==VERTICAL) {
    h=(2*w)/3;
  }
  else {
    w=h;
  }
}

int Chart::getZoom() {
  int max_stored=0;
	
  int *buf=buffer;
  for (x=0; x<w; x++) {
    if (*buf>max_stored) {
      max_stored=*buf;
    }
    buf++;
  }
  int zoom=(int)(max_stored/valuePerZoom)+1;
  if (zoom>MAX_ZOOM) {
    zoom=MAX_ZOOM;
  }

  return zoom;
}

	

void Chart::drawChart() {
  dprintf("in Chart::drawChart\n");

  int y=h;
  int num=0;

  int x,val;
  XSegment lines[SEGMENTS_PER_DRAW];

  int zoom=getZoom();
  int scale=zoom*valuePerZoom;
	
  XSegment *l=lines;
  for (x=0; x<w; x++) {
    val=buffer[(store+x)%w];
    if (val>0) {
      l->x1=l->x2=x;
      l->y1=y;
      l->y2=y - (val*h/scale);
      l++;
      num++;
      if (num==SEGMENTS_PER_DRAW) {
        XDrawSegments(display, win, chartGC, lines, num);
        l=lines;
        num=0;
      }
				
    }
  }
  if (num>0) {
    XDrawSegments(display, win, chartGC, lines, num);
  }
  if (zoom>1) {
    int i;
    l=lines;
    num=0;
    for (i=1; i<zoom; i++) {
      y=h-(h*i/zoom);
      l->x1=0;
      l->y1=y;
      l->x2=w;
      l->y2=y;
      l++;
      num++;
    }
    XDrawSegments(display, win, zoomGC, lines, num);
  }		
}

void Chart::setValue(int value) {
  store=(store+1)%w;
  int pos=(store+(w-1))%w;
  dprintf("in Chart::setValue: pos=%d value=%d\n",pos,value);
	
  int *ptr=buffer+pos;
  *ptr=value;
}

void Chart::prepare() {
  dprintf("in Chart::prepare\n");

  static int i=120;
//	i=((i+7)*3)%(valuePerZoom*3);
  i=(i+7);
  setValue(i);
}

void Chart::paint() {
  dprintf("in Chart::paint\n");
	
  drawChart();
}

void Chart::update() {
  dprintf("in Chart::update\n");
	
  drawChart();
}
