// $Id: Container.h,v 1.21 2001/04/28 17:51:31 rob Exp $

#ifndef CONTAINER_H
#define CONTAINER_H

#include "config.h"
#include "debug.h"

#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <map>

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>

#ifdef USE_DPMS
extern "C" {
#include <X11/extensions/dpms.h>
}
#endif

#define MW_EVENTS   (ExposureMask|StructureNotifyMask|ButtonPressMask|	\
	KeyPressMask|EnterWindowMask|LeaveWindowMask|PropertyChangeMask)

#define DEFAULT_FONT "-schumacher-clean-*-*-*-*-6-*-*-*-*-*-*-*"

class Container;
typedef map<Window,Container *> WindowMap;


class Container {
    friend class RowManager; // so RowManager can access x,y,w,h
	
    static bool initializedX;
    static string displayName;

    static string borderColorName;
    static string backgroundColorName;
    static string foregroundColorName;
    static string defaultFontName;

    string name;		// internal name of this winder
	
  protected:
    static const int BORDER_WIDTH=1;
    static Atom wm_del_win;
    static Colormap colormap;
    static WindowMap windowToContainer;
	
    Container *parent;		// if parent == NULL, we are a root window
    Container *firstChild;	// first window in the list, if contains other Containers
    Container *lastChild;	// last window in the list, if contains other Containers
    Container *nextSibling;	// next window in the list
    Container *prevSibling;	// previous window in the list

#ifdef USE_TOOLTIP
    Container *tooltip;
    Container *tooltipRelative;
#endif
    bool mapWhenRealized; // defaults to true
	
    int x,y;
    short w,h;

    Window win;
    XFontStruct *font;
    string font_name;
    int font_height, font_ascent, font_width;
	
    XColor border, background, foreground;
    GC drawGC,eraseGC;
	
  public:
    static Display *display;
    static Window root;
    static int screen;		// the X screen number
    static const int VERTICAL=0;
    static const int HORIZONTAL=1;

    Container();
    Container(int w, int h);
    Container(Container *parent);
    Container(Container *parent, int w, int h);
    Container(int x, int y, int w, int h);
    Container(Container *parent, int x, int y, int w, int h);
    virtual ~Container();

    virtual void realize();
    virtual void resize();
    virtual void map();
    virtual void map(Window relative, int w);
    virtual void unmap();
    void startUnmapped();
    virtual void paint();
    virtual void erase();

#ifdef USE_TOOLTIP
    void setTooltip(Container *t);
    void setTooltip(Container *t, Container *relative);
#endif
    void drawCenteredText(GC gc,int x_center, int baseline, char *str, int count);

    virtual void handleButtonPress(XButtonEvent *e);
    virtual void handleMouseMotion(XMotionEvent *e);
    virtual void handleKeyPress(XKeyEvent *e);
    virtual void handleSelectionRequest(XSelectionRequestEvent *e);
    virtual void handleSelectionClear(XSelectionClearEvent *e);
    virtual void handleEnterNotify(XCrossingEvent *e);
    virtual void handleLeaveNotify(XCrossingEvent *e);
    char *toString();
    void setParent(Container *w);
    virtual void setLayout(Container *w);
    virtual void setPreferredSize(int dir);
    static void setDefaultBackground(char *color) {
      backgroundColorName=color;
    }
    static void setDefaultForeground(char *color) {
      foregroundColorName=color;
    }
    static void setDefaultBorderColor(char *color) {
      borderColorName=color;
    }
    static void setDefaultFont(char *font) {
      defaultFontName=font;
    }
	
    static void setDisplay(char *name) {
      displayName=name;
    }
    static const char *getDisplay() {
      return displayName.c_str();
    }
    Window getWindow() {
      return win;
    }
	
      /** Find out if the monitor is on, or if it is powered down (or
       * if the screensaver is running)
       * @returns true if the monitor is displaying useful stuff;
       * false if it is powered down in energy saver mode (or the
       * screen saver is running)
       */
    static bool isScreenOn();

    Container *getFirstChild();
    Container *getNextSibling();

  protected:
    static Container *getContainerByEvent(XEvent *e);
    static void getColor(XColor * xc, const char *xc_string);
    GC getGC(const char *colorName);
    void _init();
    void createWindow();
    void Container::getColors();
    static void staticInit();
};

#endif // CONTAINER_H
