// $Id: Disk.cc,v 1.7 2001/03/03 22:02:34 rob Exp $

#include "Disk.h"

Disk::Disk(Container *parent, int tc, int deviceNum): DualBarGraph(parent,tc) {
  dprintf("in Disk::Disk\n");

  scaleFactor=2;
	
  setRange(0,w*scaleFactor); // one pixel for every scaleFactor packets
  char label[8];
  sprintf(label,"ide%d",deviceNum);
  setLabel(label);

    // get a baseline on the first read of the device status
  lastR=-1;
  lastW=-1;
  readCount=writeCount=0;

  this->deviceNum=deviceNum;
}

void Disk::prepare() {
  dprintf("in Disk::prepare (%d)\n",deviceNum);

  getDiskInfo();
  bar.cur=readCount;
  bar2.cur=writeCount;
}

// Set the two variables readCount & writeCount with the number of disk
// reads and writes since the last time.
void Disk::getDiskInfo() {
  char buf[1024];
	
  int fd = open("/proc/stat", O_RDONLY);
  if (fd>0) {
    ssize_t len=read(fd,buf,sizeof(buf)-1);
    buf[len]=(char)0;
    close(fd);

    char *str=strstr(buf,"disk_rio");
    if (str) {
      long r[4];
			
      sscanf(str,"disk_rio %ld %ld %ld %ld",&r[0],&r[1],&r[2],&r[3]);
      dprintf(" r=%ld lastR=%ld\n",r[deviceNum],lastR);
      if (lastR>0) {
        readCount=(int)(r[deviceNum]-lastR);
                  
          // for display purposes, make sure that any activity
          // less than scaleFactor is still shown as a single-pixel
          // bar.
        if (readCount>0 && readCount<scaleFactor) readCount=scaleFactor;
      }
      lastR=r[deviceNum];
    }
    else {
      dprintf("Disk: bad string search\n");
    }
		
    str=strstr(buf,"disk_wio");
    if (str) {
      long w[4];
      sscanf(str,"disk_wio %ld %ld %ld %ld",&w[0],&w[1],&w[2],&w[3]);
      dprintf(" w=%ld lastW=%ld\n",w[deviceNum],lastW);
      if (lastW>0) {
        writeCount=(int)(w[deviceNum]-lastW);
        if (writeCount>0 && writeCount<scaleFactor) writeCount=scaleFactor;
      }
      lastW=w[deviceNum];
    }
    else {
      dprintf("Disk: bad string search\n");
    }
  }
  else {
    readCount=writeCount=0;
  }
}

void Disk::create(Container *parent, int tc) {
  char name[32];

  strcpy(name,"/proc/ide/hd");
  int letterPos=strlen(name);
  char letter='a';
	
  while (letter<'e') {
    name[letterPos]=letter;
    name[letterPos+1]=(char)0;
    DIR *tmp=opendir(name);
    if (tmp) {
      closedir(tmp);
			
      dprintf("Disk::create: found IDE drive %c\n",letter);

        // ignore return value because we never destroy
      new Disk(parent,tc,letter-'a');
    }
    letter++;
  }
}
