// $Id: Manager.cc,v 1.14 2001/03/03 22:02:34 rob Exp $

/*
  This is a subclass of Container for main application windows only.
*/
#include "Manager.h"

propertyNotifyCallback_t Manager::propertyNotifyCallback=(propertyNotifyCallback_t)NULL;


Manager::Manager(char *geometry, int argc, char **argv, char *app_name, char *app_class): Container() {
  dprintf("in Manager::Manager\n");
  _init(geometry,argc,argv,app_name,app_class);
}

Manager::Manager(int w, int h, char *geometry, int argc, char **argv, char *app_name, char *app_class): Container() {
  dprintf("in Manager::Manager\n");
  this->w=w;
  this->h=h;
  _init(geometry,argc,argv,app_name,app_class);
}

Manager::Manager(int x, int y, int w, int h, char *geometry, int argc, char **argv, char *app_name, char *app_class): Container() {
  dprintf("in Manager::Manager\n");
  this->x=x;
  this->y=y;
  this->w=w;
  this->h=h;
  _init(geometry,argc,argv,app_name,app_class);
}

void Manager::_init(char *geometry, int argc, char **argv, char *app_name, char *app_class) 
{
  XWMHints wmhints;

  sizehints=XAllocSizeHints();
	
  width_req=height_req=0;
  geometry_flags = XParseGeometry(geometry, &x_req, &y_req, &width_req, &height_req);
  dprintf("XParseGeometry: flags=%d geometry=%s x=%d y=%d w=%d h=%d\n",geometry_flags,geometry,x_req,y_req,width_req,height_req);
	
  XTextProperty wname, iname;
  XStringListToTextProperty(&app_name, 1, &wname);
  XStringListToTextProperty(&app_name, 1, &iname);

  XClassHint class1;
  class1.res_name = app_name;
  class1.res_class = app_class;
  wmhints.input = True;
  wmhints.flags = InputHint;
  XSetWMProperties(display, win, &wname, &iname,
    argv, argc, NULL, &wmhints, &class1);

    /* want to accept the delete window protocol on root windows */
  XSetWMProtocols(display, win, &wm_del_win, 1);
}

void Manager::resize() 
{
  sizehints->flags=PSize|USSize|PMinSize|PMaxSize|PResizeInc|PBaseSize|PWinGravity;
	
  sizehints->win_gravity = NorthWestGravity;
  sizehints->height = sizehints->base_height = sizehints->min_height =
    sizehints->max_height = h;
  sizehints->width = sizehints->base_width = sizehints->min_width = 
    sizehints->max_width = w;
  sizehints->width_inc = sizehints->height_inc = 0;

  if (geometry_flags & XValue) {
    if (geometry_flags & XNegative) {
      x = DisplayWidth(display, screen) + x_req - w - 2 * BORDER_WIDTH;
      sizehints->win_gravity = NorthEastGravity;
    }
    else {
      x=x_req;
    }
    sizehints->flags|=PPosition|USPosition;
  }
  if (geometry_flags & YValue) {
    if (geometry_flags & YNegative) {
      y = DisplayHeight(display, screen) + y_req - h - 2 * BORDER_WIDTH;
      sizehints->win_gravity = SouthWestGravity;
      if ((geometry_flags & XValue) && (geometry_flags & XNegative))
        sizehints->win_gravity = SouthEastGravity;
    }
    else {
      y=y_req;
    }
    sizehints->flags|=PPosition|USPosition;
  }
  sizehints->x=x;
  sizehints->y=y;

  dprintf("Manager::resize: x=%d y=%d h=%d w=%d\n",x,y,w,h);
	
  XMoveResizeWindow(display,win,x,y,w,h);
  XSetWMNormalHints(display,win,sizehints);
}

void Manager::invalidate() 
{
	
}

void Manager::doLayout() 
{
  invalidate();
  Container *w=getFirstChild();
  while (w) {
    setLayout(w);
    w->resize();
    w=w->getNextSibling();
  }
}

void Manager::setLayout(Container *target) 
{
	
}

void Manager::setPreferredSize() 
{

}


void Manager::realize() 
{
  doLayout();
  setPreferredSize();
  resize();
  Container::realize();
  Container *w=getFirstChild();
  while (w) {
    w->realize();
    w=w->getNextSibling();
  }
}

void Manager::paintAll() 
{
  Container *w=getFirstChild();
  while (w) {
    w->erase();
    w->paint();
    w=w->getNextSibling();
  }
}
	

void Manager::registerPropertyNotifyHandler(propertyNotifyCallback_t callback) {
  propertyNotifyCallback=callback;
}

void Manager::firePropertyNotifyHandler(XPropertyEvent *e) {
  if (propertyNotifyCallback) {
    (propertyNotifyCallback)(e);
  }
}

void Manager::processEvent(XEvent *event) 
{
  Container *w;
	
  switch (event->type) {
		
      /* check for delete window requests */
    case ClientMessage:
      if ((event->xclient.format == 32) &&
        ((Atom)event->xclient.data.l[0] == wm_del_win)) {
        dprintf("wm_del_win %s\n",w->toString());
				//if (event->xany.window == win)
        exit(0);
      }
      break;

        /* need to re-draw a window */
    case Expose:
    case GraphicsExpose:
      w=getContainerByEvent(event);
      dprintf("Expose %s\n",w->toString());
      w->paint();
      break;

        /* window has been re-sized */
    case ConfigureNotify:
      dprintf("ConfigureNotify\n");
      break;

    case EnterNotify:
      dprintf("EnterNotify\n");
      w=getContainerByEvent(event);
      if (w) {
        w->handleEnterNotify(&event->xcrossing);
      }
      break;

    case LeaveNotify:
      dprintf("LeaveNotify\n");
      w=getContainerByEvent(event);
      if (w) {
        w->handleLeaveNotify(&event->xcrossing);
      }
      break;

        /* cut buffer changes should be caught here */
    case PropertyNotify:
      firePropertyNotifyHandler(&event->xproperty);
      break;

    case SelectionRequest:
      w=getContainerByEvent(event);
      dprintf("SelectionRequest\n");
      w->handleSelectionRequest(&event->xselectionrequest);
      break;

    case SelectionClear:
      w=getContainerByEvent(event);
      dprintf("SelectionClear\n");
      w->handleSelectionClear(&event->xselectionclear);
      break;

        /* button presses in cut buffer windows */
    case ButtonPress:
      w=getContainerByEvent(event);
      dprintf("ButtonPress %s\n",w->toString());
      w->handleButtonPress(&event->xbutton);
      break;

    case MotionNotify:
      w=getContainerByEvent(event);
      dprintf("MotionNotify %s\n",w->toString());
      w->handleMouseMotion(&event->xmotion);
      break;

    case KeyPress:
      w=getContainerByEvent(event);
      dprintf("KeyPress %s\n",w->toString());
      w->handleKeyPress(&event->xkey);
      break;
  }
}

void Manager::eventLoop() 
{
  for (;;) {
    XEvent event;

    XNextEvent(display, &event);
//		dprintf("Event: ");
    processEvent(&event);
  }
}

