// $Id: NetPackets.cc,v 1.8 2001/03/04 03:26:33 rob Exp $

#include "NetPackets.h"

NetPackets::NetPackets(Container *parent, int tc, char *device): DualBarGraph(parent,tc) {
  dprintf("in NetPackets::NetPackets\n");

  scaleFactor=2;
	
  setRange(0,w*scaleFactor); // one pixel for every scaleFactor packets
  setLabel(device);

    // get a baseline on the first read of the device status
  lastTx=-1;
  lastRx=-1;
  txCount=rxCount=0;
	
  deviceName=device;
  deviceName+=":";
}

void NetPackets::prepare() {
  dprintf("in NetPackets::prepare (%s)\n",deviceName.c_str());

  getNetInfo();
  bar.cur=rxCount;
  bar2.cur=txCount;
}

// get the number of transmitted and recieved packets on the interface.
void NetPackets::getNetInfo() {
  char buf[1024];
	
  int fd = open("/proc/net/dev", O_RDONLY);
  if (fd>0) {
    ssize_t len=read(fd,buf,sizeof(buf)-1);
    buf[len]=(char)0;
    close(fd);

    char *str=strstr(buf,deviceName.c_str());
    if (str) {
      long dum, tx, rx;
			
      str+=deviceName.size();
        //132853000  271934 0 0 0 0 0 0   7800952   87243
      sscanf(str,"%ld %ld %ld %ld %ld %ld %ld %ld %ld %ld",
        &dum,&rx,&dum,&dum,&dum,&dum,&dum,&dum,&dum,&tx);
          
      if (lastTx>0) {
        txCount=(int)(tx-lastTx);
                  
        dprintf("NetPackets::prepare  %s tx packet diff=%d\n",
          deviceName.c_str(),txCount);

          // for display purposes, make sure that any activity
          // less than scaleFactor is still shown as a single-pixel
          // bar.
        if (txCount>0 && txCount<scaleFactor) txCount=scaleFactor;
      }
      lastTx=tx;
      if (lastRx>0) {
        rxCount=(int)(rx-lastRx);
                  
        dprintf("NetPackets::prepare  %s rx packet diff=%d\n",
          deviceName.c_str(),rxCount);
        if (rxCount>0 && rxCount<scaleFactor) rxCount=scaleFactor;
      }
      lastRx=rx;
    }
    else {
      dprintf("NetPackets: bad string search\n");
    }
  }
}

void NetPackets::create(Container *parent, int tc) {
  char buf[1024];
	
  int fd = open("/proc/net/dev", O_RDONLY);
  if (fd>0) {
    ssize_t len=read(fd,buf,sizeof(buf)-1);
    buf[len]=(char)0;
    close(fd);

    char *str=buf;
    while ((str=strstr(str,":"))!=NULL) {
      char interface[16];
      int len=0;
      str--;
      while (str>=buf && isalnum(*str)) str--,len++;
      str++;
      strncpy(interface,str,len);
      interface[len]=(char)0;
			
      dprintf("NetPackets::create: found interface %s\n",interface);

        // ignore return value because we never destroy
      new NetPackets(parent,tc,interface);

      str+=len+1;
    }
  }
}
