// $Id: TextBox.cc,v 1.4 2001/03/03 22:02:34 rob Exp $

#include "TextBox.h"

TextBox::TextBox(Container *parent): Container(parent) {
  dprintf("in TextBox::TextBox\n");

  this->rows=2;
  this->highlighted=false;
  this->text=NULL;
  this->length=0;
}

TextBox::TextBox(Container *parent, int rows): Container(parent) {
  dprintf("in TextBox::TextBox\n");

  this->rows=rows;
  this->highlighted=false;
  this->text=NULL;
  this->length=0;
}

void TextBox::setPreferredSize(int dir) {
  dprintf("in TextBox::setPreferredSize\n");

  if (dir==VERTICAL) {
    h=font_height*rows;
  }
  else {
    w=h;
  }

    // handle variable width fonts by using the narrowest character when
    // determining the number of columns to display.
  int min_width=font->min_bounds.width;
  if (min_width<1) min_width=1;
  cols=w/min_width;
  if (cols*min_width < w) cols++;
}

void TextBox::setText(char *text) {
  this->text=text;
  this->length=strlen(text);
}

void TextBox::setText(char *text, int length) {
  this->text=text;
  this->length=length;
}

void TextBox::setHighlight(bool highlighted) {
  this->highlighted=highlighted;
}

void TextBox::drawText() {
  dprintf("in TextBox::drawText\n");

  int x = 0;
  int	y = 0;
  char *c = text;
  int	len = length;

  while (y < h && len > 0) {
    char temp[256];
		
    int count = 0;
    bool gotLine = false;
    while (count < cols && len > 0 && !gotLine) {
      if (*c != '\n') {
        temp[count++] = *c;
      }
      else {
        gotLine = true;
      }
      len--;
      c++;
    }
    while (!gotLine && len > 0) {
      if (*c == '\n')
        gotLine = true;
      len--;
      c++;
    }
		
		
    if (count > 0) {
      if (highlighted)
        XDrawImageString(display, win, eraseGC,
          x, y + font_ascent, temp, count);
      else
        XDrawImageString(display, win, drawGC,
          x, y + font_ascent, temp, count);
    }
    y += font_height;
  }
}

void TextBox::paint() {
  dprintf("in TextBox::paint\n");
	
  drawText();
}

void TextBox::update() {
  dprintf("in TextBox::update\n");
	
  drawText();
}
