// $Id: TimeManager.cc,v 1.11 2001/04/28 17:51:31 rob Exp $

/*
  This is a subclass of Container for main application windows only.
*/

#include "TimeManager.h"
#ifdef USE_SELECTOR
#include "Selector.h"
#endif

int TimeManager::tickHappened=0;
int TimeManager::secondsPerTick=1;
int TimeManager::readPipe=-1;
int TimeManager::writePipe=-1;

TimeManager::TimeManager(char *geometry, int argc, char **argv, char *app_name, char *app_class): Manager(geometry,argc,argv,app_name,app_class) {
  dprintf("in TimeManager::TimeManager\n");
}

void TimeManager::incrementAll() 
{
  dprintf("in TimeManager::incrementAll\n");

  Container *w=getFirstChild();
  while (w) {

// Have to use the dynamic cast here because I'm not limiting the
// managed containers to only hold TimedContainers.  If the base class
// of all Manager classes was TimedContainer instead of Container, I
// wouldn't need this cast.  (Well, not strictly true.  I'd also have
// to do some other things, like move around the call to
// parent->setLayout and createWindow to the TimedContainer
// constructors, but you get the idea.)
    TimedContainer *t=dynamic_cast<TimedContainer*>(w);
    if (t) {
      dprintf("  found %s is a TimedContainer\n",t->toString());
      if (t->incrementTick()) {
        t->erase();
        t->prepare();
        t->paint();
      }
    }
    else {
      dprintf("  found %s is NOT a TimedContainer\n",w->toString());
    }
		
    w=w->getNextSibling();
  }
}

// This is the signal handler for SIGALRM that the program uses to
// update the display every specified interval.  A signal handler
// shouldn't call an X event during the signal processing stage, so
// the only thing that this does is write a byte to a pipe.  Writing
// to pipes is async-safe (unlike generating X events), and so the
// main loop uses select to determine if there is a byte waiting on
// the pipe.  A byte waiting means that the alarm has happened, so it
// is time to increment all of the TimedContainers.
void TimeManager::sighandler(int sig) {
  dprintf("in TimeManager::sighandler\n");

  write(writePipe,"s",1);
  alarm(secondsPerTick);
}

void TimeManager::eventLoop(int secondsPerTick) {
  XEvent event;
  this->secondsPerTick=secondsPerTick;

  int pipe_fds[2];
  if (pipe(pipe_fds)<0) {
      // initially, I had an alternate loop based on polling if you
      // can't open up a pipe.  But, something is probably very wrong
      // with your system if the pipe can't be opened.  It probably
      // wouldn't make much difference if this were able to run
      // without pipes, because if you are out of file descriptors you
      // wouldn't be able to run anything else after this program that
      // used pipes.  So, it wasn't worth the effort to keep two event
      // loops around.
    printf("Seriously bad: couldn't open pipe for signal handler.\n");
    exit(1);
  }
	
  writePipe=pipe_fds[1];
  readPipe=pipe_fds[0];
	
  int fd=XConnectionNumber(display);

  int max_fd=fd;
  if (readPipe>max_fd) max_fd=readPipe;
  max_fd++;

  while (XPending(display)) {
    XNextEvent(display,&event);
    processEvent(&event);
  }

    // set up signal handler using POSIX sigaction instead of signal.
    // sigaction is cleaner inside the signal handler itself, because
    // you don't have to write the code to ignore the signal while in
    // the signal handler, and you don't have to set up the signal
    // handler again.  The signal handler set by sigaction becomes the
    // signal handler until expliticly reset.
  struct sigaction alarm_action;
  alarm_action.sa_flags = 0;
  alarm_action.sa_handler = TimeManager::sighandler;
  sigemptyset(&alarm_action.sa_mask);
  if (sigaction(SIGALRM, &alarm_action, NULL)<0) {
    perror("alarm handler");
    return;
  }
  alarm(secondsPerTick);

  XFlush(display);

    // main loop.  We wait on an I/O event on the X file descriptor,
    // or the signal-handling pipe.
  for (;;) {
    fd_set fds;
		
    FD_ZERO(&fds);
    FD_SET(fd,&fds);
    FD_SET(readPipe,&fds);
#ifdef USE_SELECTOR
    bool extra=Selector::setFileDescriptors(&max_fd,&fds);
#endif
    int ret=select(max_fd,&fds,NULL,NULL,NULL);
    dprintf("ret=%d  X waiting=%d  pipe waiting=%d\n",ret,
      (int)FD_ISSET(fd,&fds),(int)FD_ISSET(readPipe,&fds));
    if (ret>0) {
      if (FD_ISSET(readPipe,&fds)) {
        char tmp;
        read(readPipe,&tmp,1);
        dprintf("tickHappened: \n");
        if (isScreenOn()) incrementAll();
//        else cerr << "  screen is blanked; no drawing." << endl;
      }
#ifdef USE_SELECTOR
      if (extra) {
        Selector::checkFileDescriptors(&fds);
      }
#endif
    }

      // We have received some event, and we don't care what it is.
      // Process any X events if there are any.  We still process
      // events even though the screen might be blanked because we
      // want to immediately refresh the window when the screen is
      // unblanked.
    while (XPending(display)) {
      XNextEvent(display, &event);
      dprintf("Event: \n");
      processEvent(&event);
    }
  }
}
