// $Id: TimedContainer.cc,v 1.4 2001/03/03 22:02:34 rob Exp $

#include "TimedContainer.h"

TimedContainer::TimedContainer(Container *parent, int tc): Container(parent) {
  dprintf("in TimedContainer::TimedContainer\n");
  _init(tc);
}

void TimedContainer::_init(int tc) {
  tick=0;
  tickCount=tc;
}

void TimedContainer::realize() {
  dprintf("in TimedContainer::realize\n");

  prepare();
  Container::realize();
}

// Another virtual function designed to be called by the Container
// class.  Width or height can be adjusted depending on the dir flag.  If
// dir is VERTICAL, the container is expecting the height to be adjusted.
void TimedContainer::setPreferredSize(int dir) 
{
  dprintf("in TimedContainer::setPreferredSize\n");

  if (dir==VERTICAL) {
    h=10; // default to rectangular so we can debug it.
  }
  else {
    w=10;
  }
}

int TimedContainer::getTick() {
  return tick;
}

int TimedContainer::getTickCount() {
  return tickCount;
}

bool TimedContainer::incrementTick() {
  tick++;
  if (tick>=tickCount) {
    tick=0;
    return true;
  }
  return false;
}

bool TimedContainer::setTick(int t) {
  tick=t;
  if (tick>=tickCount){
    tick=0;
    return true;
  }
  return false;
}

	
bool TimedContainer::setTickCount(int tc) {
  tickCount=tc;
  if (tick>=tickCount){
    tick=0;
    return true;
  }
  return false;
}


// default method that is called by TimeManager when the tick count
// is reached.  Should be overridden in subclasses, since this doesn't
// do too much here.
void TimedContainer::prepare() {

}


// default paint.  Should be overridden.
void TimedContainer::paint()
{
  dprintf("in TimedContainer::paint\n");

  XSegment lines[100];

  lines[0].x1=w>>1;
  lines[0].y1=0;
  lines[0].x2=w>>1;
  lines[0].y2=h-1;
  lines[1].x1=0;
  lines[1].y1=h>>1;
  lines[1].x2=w-1;
  lines[1].y2=h>>1;
	
  XDrawSegments(display, win, drawGC, lines, 2);
}

// This is called when the container needs updating, but a full redraw
// is not needed.
void TimedContainer::update() 
{
	
}
