// $Id: Tooltip.cc,v 1.5 2001/03/03 22:02:34 rob Exp $

#include "Tooltip.h"

// This constructor contains an ugly hack because apparently C++ won't call
// overridden methods in derived classes during a constructor.  I suppose that
// this is actually a Good Thing from a language design standpoint, because
// methods in a derived class probably depend on stuff being initialized in
// the constructor of the derived class.  But, this makes this particular
// instance more difficult because the Container constructor then automatically
// constructs a window for the Container, and doesn't look at the overridden
// Tooltip::createWindow.  So, I have to call it myself here.

Tooltip::Tooltip(): Container(10,10) {
  dprintf("in Tooltip::Tooltip\n");

  XDestroyWindow(display,win); // remove window created by Container
  createWindow(); // create new one with correct attributes for tooltip
  realize();

  sized=false;
}

void Tooltip::createWindow() {
  dprintf("in Tooltip::createWindow\n");
  XSetWindowAttributes attr;
  unsigned long valuemask;
	
  valuemask=CWOverrideRedirect | CWEventMask | CWBackPixel | CWBorderPixel;
  attr.override_redirect = True; // tell window manager not to decorate
  attr.event_mask=TOOLTIP_EVENTS;
  attr.background_pixel=background.pixel;
  attr.border_pixel=border.pixel;
  win = XCreateWindow(display, root, x, y, w, h,
    BORDER_WIDTH, CopyFromParent, InputOutput, CopyFromParent,
    valuemask, &attr);

    // turn off input requests so window manager never tries to bring
    // the tooltip window to the front.
  XWMHints wmhints;
  wmhints.input = False;
  wmhints.flags = InputHint;
  XSetWMHints(display, win, &wmhints);
	
  windowToContainer[win]=this;

  mapWhenRealized=false;
}

void Tooltip::map(Window relative, int parent_width) {
  if (!sized) {
    setPreferredSize(VERTICAL);
    sized=true;
  }
	
  int rootx,rooty;
  Window dum;
	
  XTranslateCoordinates(display,relative,root,0,0,&rootx,&rooty,&dum);

    // try to position the tooltip to the left of the window
  x=rootx-w - BORDER_WIDTH*2;
  if (x<0) {
      // nope.  Off screen, put it to the right
    x=rootx+parent_width;
  }
  y=rooty;
	
  resize();
		
    // bring the newly mapped window to the top of the stacking order.
  XMapRaised(display,win);
}

